/**************************************************************************************************/

/*
* File: main.c
* Author: Roman Lysecky
* Date: 04 September 2012
*
* Description: Program to create a dynamically allocated array of a user specified size, and
* remove all entries that match the maximum value.
*
*/

/**************************************************************************************************/

#include <stdio.h>
#include <stdlib.h>

/**************************************************************************************************/

/*
* Find the index within an array matching the provided key .
*
* RETURN: index matching of the first entry within the array equal to the key
*         -1 otherwise
*/
int find_key(int *dataValues, int numEntries, int key) {

   int i;

   /* check to ensure myArray is not NULL */
   if (dataValues == NULL ) return -1;

   /* for all entires in array */
   /*     if entry matches the key, return index */
   for ( i=0; i<numEntries; i++ ) {
      if ( *(dataValues+i) == key ) return i;
   }

   /* if key if not found, return -1 */
   return -1;
}

/**************************************************************************************************/

/*
* Find the maximum values within the input array and store the result to
* memory location pointed to by max_ptr.
*
* RETURN: 1 if successful, 0 otherwise
*/
int find_max(int *dataValues, int numEntries, int *max) {
   int i;
   int largestNum;

   if ( numEntries == 0 ) return 0;
   largestNum = dataValues[0];     /* initialize largest number fo first elements */

   i = 0;                          /* we could start with 1 in this example */
   while ( i < numEntries ) {
      if ( dataValues[i] > largestNum ) {
         largestNum = dataValues[i];
      }
      i++;
   }

   // assign max
   *max = largestNum;

   return 1;
}

/**************************************************************************************************/

int
   main(void)
{
   int i;
   int max;
   int userNumEntries;
   int *new_array;
   int max_location;

   /* Read number of elements from user and assigne to var userNumEntries
   * Check to ensure a valid input was given
   */
   userNumEntries = 0;
   printf("Enter the number of entries for the array: ");
   scanf("%d", &userNumEntries);
   if ( userNumEntries <= 0 ) {
      printf("Nothing to do here. Moving along.\n");
      return -1;
   }

   /*
   * Dynamically allocate the array with userNumEntries integers
   * Check to ensure the memory was allocated
   *
   */
   new_array = (int*)malloc(userNumEntries*sizeof(int));
   if ( new_array == NULL ) {
      printf("Error: Could not allocate memory for new_array.\n");
      return -1;
   }

   /*
   * Initialize array items with user entered values
   *

   * for userNumEntires
   *    ask  user to enter an integer value
   *    check to ensure the integer value is valid
   */
   for ( i=0; i<userNumEntries; i++ ) {
      printf("new_array[%d] = ", i);
      if ( scanf("%d", &(new_array[i])) != 1 ) {
         printf("Invalid input.\n");
         return -1;
      }
   }

   printf("\Before Max Removal:\n");
   for ( i=0; i<userNumEntries; i++ ) {
      printf("new_array[%d] = %d\n", i, new_array[i]);
   }

   /*
   * Find the max value within the array
   */
   if ( find_max(new_array, userNumEntries, &max) ) {

      /* print the max */
      printf("\nMax Value = %d\n", max);

      /*
      * If max if successfully found,
      * repeatedly find the index of the item equal to max
      * Remove the value, shifting the contents, and resizeing the array
      */

      /* while the max value is found, remove the max value */
      while ( find_key(new_array, userNumEntries, max) != -1 ) {

         /* get the location of the first item matching max */
         max_location = find_key(new_array, userNumEntries, max);

         /* from max_location to entry before the end of array */
         for ( i=max_location; i<userNumEntries-1; i++ ) {
            /* copy the next array entry to the current array entry */
            new_array[i] = new_array[i+1];
         }
         
         /* reduce the number of user entries */
         userNumEntries--;

         /* if the array will become empty, free the memory for the array 
         * and set the pointer to NULL */
         if( userNumEntries == 0 ) {
            free(new_array);
            new_array = NULL;
            break;
         }
         else {
         /* adjust the size of the array (what if the array will be empty??) */
            new_array = (int*)realloc(new_array, userNumEntries*sizeof(int));
            if ( new_array == NULL ) {
               printf("Error: Failed to reallocate memory for new_array\n");
               return -1;
            }
         }
      }
   }

   printf("\nAfter Max Removal:\n");
   for ( i=0; i<userNumEntries; i++ ) {
      printf("new_array[%d] = %d\n", i, new_array[i]);
   }

   /* all done, free the array and set the pointer to NULL */
   free(new_array);
   new_array = NULL;

   return 0;
}

/**************************************************************************************************/



